#! /usr/bin/env python3

import csv
import os
import os.path
import requests
import urllib

_wikidata = 'http://www.wikidata.org/w/api.php?'
_commons = 'http://commons.wikimedia.org/w/api.php?'
_wmflabs = 'http://tools.wmflabs.org/wikidata-todo/download_query.php?'

_countries = '''<?xml version="1.0" encoding="UTF-8"?>
<countries>
%s</countries>
'''
_country = '<country name="%s" iso-3166="%s" itu-e164="%s" />'

_xmls = 'countries'
_flags = 'flags'

_properties = {
	'itu-e164': 'P474',
	'iso-3166': 'P901',
	'flag': 'P41',
	'continent': 'P30',
	'type': 'P31'
}

_entities = {
	'country': 'Q6256'
}

class Properties:
	def __init__(self, claims):
		self.__claims = claims

	def __getitem__(self, claim):
		return self.__claims[_properties[claim]][0]['mainsnak']['datavalue']['value']

def request(site, **query):
	return requests.get(site + urllib.urlencode(query))

def search(property, entity):
	return request(_wmflabs,
		q='CLAIM[%s:%s]' % (property[1:], entity[1:])
	).content

def getProperties(country):
	return Properties(request(_wikidata,
		action='wbgetentities',
		ids=country,
		props='claims',
		languages='fr',
		format='json'
	).json['entities'].values()[0]['claims'])

def download(flag):
	return requests.get(
		request(_commons,
			action='query',
			prop='pageimages',
			titles='File:' + flag,
			format='json'
		).json['query']['pages'].values()[0]['thumbnail']['source'].replace('50px', '2000px')
	).content

for dir in [_xmls, _flags]:
	if not os.path.exists(dir):
		os.mkdir(dir)

continents = {}
for entity, country, _, _ in sorted(list(csv.reader(search(
	_properties['type'],
	_entities['country']
).split('\n')[1:-1], delimiter='\t')), cmp=lambda x, y: cmp(x[1], y[1])):
	if not (country or entity):
		continue
	print('Getting information on {} ({})'.format(country, entity))
	properties = getProperties(entity)
	try:
		continent = str(properties['continent']['numeric-id'])
		countries = continents.get(continent, '')
		iso3166 = properties['iso-3166'].lower()
		itue164 = properties['itu-e164'][1:]
		flag = properties['flag']
	except KeyError:
		continue
	countries += '\t' + _country % (country, iso3166, itue164) + '\n'
	continents[continent] = countries
	with open(_flags + '/%s.png' % iso3166, 'w') as file:
		file.write(download(flag))

for continent in continents.iteritems():
	with open(_xmls + '/%s.xml' % continent[0], 'w') as file:
		file.write(_countries % continent[1])
